@@ -21,7 +21,35 @@
 #[macro_use]
 extern crate log;
 
-use windmark::Response;
+use windmark::{returnable::CallbackContext, Response, Router};
+
+#[derive(Default)]
+struct Clicker {
+  clicks: isize,
+}
+impl windmark::Module for Clicker {
+  fn on_attach(&mut self, _: &mut Router) {
+    println!("clicker has been attached!");
+  }
+
+  fn on_pre_route(&mut self, context: CallbackContext<'_>) {
+    self.clicks += 1;
+
+    info!(
+      "clicker has been called pre-route on {} with {} clicks!",
+      context.url.path(),
+      self.clicks
+    );
+  }
+
+  fn on_post_route(&mut self, context: CallbackContext<'_>) {
+    info!(
+      "clicker has been called post-route on {} with {} clicks!",
+      context.url.path(),
+      self.clicks
+    );
+  }
+}
 
 #[windmark::main]
 async fn main() -> Result<(), Box<dyn std::error::Error>> {
@@ -29,7 +57,7 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
 
   windmark::Router::new()
     .set_private_key_file("windmark_private.pem")
-    .set_certificate_chain_file("windmark_pair.pem")
+    .set_certificate_file("windmark_public.pem")
     .enable_default_logger(true)
     .set_error_handler(Box::new(move |_| {
       error_count += 1;
@@ -38,12 +66,13 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
 
       Response::PermanentFailure("e".into())
     }))
-    .attach(|r| {
+    .attach_stateless(|r| {
       r.mount(
         "/module",
         Box::new(|_| Response::Success("This is a module!".into())),
       );
     })
+    .attach(Clicker::default())
     .set_pre_route_callback(Box::new(|stream, url, _| {
       info!(
         "accepted connection from {} to {}",
@@ -155,10 +184,20 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
         Response::PermanentRedirect("gemini://localhost/test".into())
       }),
     )
-    .mount(
-      "/file",
-      Box::new(|_| Response::SuccessFile(include_bytes!("../LICENSE"))),
-    )
+    .mount("/file", {
+      #[cfg(feature = "auto-deduce-mime")]
+      {
+        Box::new(|_| Response::SuccessFile(include_bytes!("../LICENSE")))
+      }
+
+      #[cfg(not(feature = "auto-deduce-mime"))]
+      Box::new(|_| {
+        Response::SuccessFile(
+          include_bytes!("../LICENSE"),
+          "text/plain".to_string(),
+        )
+      })
+    })
     .run()
     .await
 }
