@@ -32,11 +32,15 @@
 //! # Cargo.toml
 //!
 //! [dependencies]
-//! windmark = "0.1.3"
+//! windmark = "0.1.8"
 //! tokio = { version = "0.2.4", features = ["full"] }
 //!
 //! # If you would like to use the built-in logger (recommended)
-//! # windmark = { version = "0.1.3", features = ["logger"] }
+//! # windmark = { version = "0.1.8", features = ["logger"] }
+//!
+//! # If you would like to use the built-in MIME dedection when `Success`-ing a file
+//! # (recommended)
+//! # windmark = { version = "0.1.8", features = ["auto-deduce-mime"] }
 //! ```
 //!
 //! ### Implement a Windmark server
@@ -50,7 +54,7 @@
 //! fn main() -> Result<(), Box<dyn std::error::Error>> {
 //!   windmark::Router::new()
 //!     .set_private_key_file("windmark_private.pem")
-//!     .set_certificate_chain_file("windmark_pair.pem")
+//!     .set_certificate_chain_file("windmark_public.pem")
 //!     .mount("/", Box::new(|_| Response::Success("Hello, World!".into())))
 //!     .set_error_handler(Box::new(|_| {
 //!       Response::PermanentFailure("This route does not exist!".into())
@@ -65,6 +69,21 @@
 //! Examples can be found within the
 //! [`examples/`](https://github.com/gemrest/windmark/tree/main/examples) directory.
 //!
+//! ## Modules
+//!
+//! Modules are reusable extensions which can be procedurally mounted onto
+//! Windmark routers.
+//!
+//! [Add yours!](https://github.com/gemrest/windmark/edit/main/README.md)
+//!
+//! - [Windmark Comments](https://github.com/gemrest/windmark-comments)
+//!
+//! ## Capsules using Windmark
+//!
+//! [Add yours!](https://github.com/gemrest/windmark/edit/main/README.md)
+//!
+//! - <https://fuwn.me/>
+//!
 //! ## License
 //!
 //! This project is licensed with the
@@ -83,8 +102,9 @@
 #![recursion_limit = "128"]
 
 mod handler;
+pub mod module;
 pub mod response;
-pub(crate) mod returnable;
+pub mod returnable;
 pub mod utilities;
 
 #[macro_use]
@@ -95,6 +115,7 @@ use std::{
   sync::{Arc, Mutex},
 };
 
+pub use module::Module;
 use openssl::ssl::{self, SslAcceptor, SslMethod};
 pub use response::Response;
 pub use tokio::main;
@@ -107,26 +128,28 @@ use url::Url;
 use crate::{
   handler::{Callback, ErrorResponse, Partial, RouteResponse},
   response::to_value_set_status,
-  returnable::{ErrorContext, RouteContext},
+  returnable::{CallbackContext, ErrorContext, RouteContext},
 };
 
 /// A router which takes care of all tasks a Windmark server should handle:
 /// response generation, panics, logging, and more.
 #[derive(Clone)]
 pub struct Router {
-  routes: matchit::Router<Arc<Mutex<RouteResponse>>>,
-  error_handler: Arc<Mutex<ErrorResponse>>,
+  routes:                matchit::Router<Arc<Mutex<RouteResponse>>>,
+  error_handler:         Arc<Mutex<ErrorResponse>>,
   private_key_file_name: String,
-  certificate_chain_file_name: String,
-  header: Arc<Mutex<Partial>>,
-  footer: Arc<Mutex<Partial>>,
-  ssl_acceptor: Arc<SslAcceptor>,
+  ca_file_name:          String,
+  header:                Arc<Mutex<Partial>>,
+  footer:                Arc<Mutex<Partial>>,
+  ssl_acceptor:          Arc<SslAcceptor>,
   #[cfg(feature = "logger")]
-  default_logger: bool,
-  pre_route_callback: Arc<Mutex<Callback>>,
-  post_route_callback: Arc<Mutex<Callback>>,
-  charset: String,
-  language: String,
+  default_logger:        bool,
+  pre_route_callback:    Arc<Mutex<Callback>>,
+  post_route_callback:   Arc<Mutex<Callback>>,
+  charset:               String,
+  language:              String,
+  port:                  i32,
+  modules:               Arc<Mutex<Vec<Box<dyn Module + Send>>>>,
 }
 impl Router {
   /// Create a new `Router`
@@ -164,13 +187,10 @@ impl Router {
   /// # Examples
   ///
   /// ```rust
-  /// windmark::Router::new().set_certificate_chain_file("windmark_pair.pem");
+  /// windmark::Router::new().set_certificate_file("windmark_public.pem");
   /// ```
-  pub fn set_certificate_chain_file(
-    &mut self,
-    certificate_chain_file_name: &str,
-  ) -> &mut Self {
-    self.certificate_chain_file_name = certificate_chain_file_name.to_string();
+  pub fn set_certificate_file(&mut self, certificate_name: &str) -> &mut Self {
+    self.ca_file_name = certificate_name.to_string();
 
     self
   }
@@ -255,7 +275,7 @@ impl Router {
   /// # Examples
   ///
   /// ```rust
-  /// windmark::Router::new().run();
+  /// windmark::Router::new().run(); 
   /// ```
   ///
   /// # Panics
@@ -274,7 +294,8 @@ impl Router {
     }
 
     let acceptor = self.ssl_acceptor.clone();
-    let mut listener = tokio::net::TcpListener::bind("0.0.0.0:1965").await?;
+    let mut listener =
+      tokio::net::TcpListener::bind(format!("0.0.0.0:{}", self.port)).await?;
 
     #[cfg(feature = "logger")]
     info!("windmark is listening for connections");
@@ -311,6 +332,8 @@ impl Router {
     let mut buffer = [0u8; 1024];
     let mut url = Url::parse("gemini://fuwn.me/")?;
     let mut response_status = 0;
+    #[cfg(not(feature = "auto-deduce-mime"))]
+    let mut response_mime_type = "".to_string();
     let mut footer = String::new();
     let mut header = String::new();
     let content;
@@ -325,7 +348,17 @@ impl Router {
       }
     }
 
-    let route = self.routes.at(url.path());
+    let route = &mut self.routes.at(url.path());
+
+    for module in &mut *self.modules.lock().unwrap() {
+      module.on_pre_route(CallbackContext::new(stream.get_ref(), &url, {
+        if let Ok(route) = &route {
+          Some(&route.params)
+        } else {
+          None
+        }
+      }));
+    }
 
     (*self.pre_route_callback).lock().unwrap().call_mut((
       stream.get_ref(),
@@ -370,6 +403,8 @@ impl Router {
             &route.params,
           ),)),
           &mut response_status,
+          #[cfg(not(feature = "auto-deduce-mime"))]
+          &mut response_mime_type,
         )
       };
     } else {
@@ -379,6 +414,8 @@ impl Router {
           .unwrap()
           .call_mut((ErrorContext::new(stream.get_ref(), &url),)),
         &mut response_status,
+        #[cfg(not(feature = "auto-deduce-mime"))]
+        &mut response_mime_type,
       );
     }
 
@@ -397,7 +434,10 @@ impl Router {
                 " text/gemini; charset={}; lang={}",
                 self.charset, self.language
               ),
+            #[cfg(feature = "auto-deduce-mime")]
             21 => tree_magic::from_u8(&*content.as_bytes()),
+            #[cfg(not(feature = "auto-deduce-mime"))]
+            21 => response_mime_type,
             _ => (&*content).to_string(),
           },
           match response_status {
@@ -410,6 +450,16 @@ impl Router {
       )
       .await?;
 
+    for module in &mut *self.modules.lock().unwrap() {
+      module.on_post_route(CallbackContext::new(stream.get_ref(), &url, {
+        if let Ok(route) = &route {
+          Some(&route.params)
+        } else {
+          None
+        }
+      }));
+    }
+
     (*self.post_route_callback).lock().unwrap().call_mut((
       stream.get_ref(),
       &url,
@@ -434,7 +484,10 @@ impl Router {
       &self.private_key_file_name,
       ssl::SslFiletype::PEM,
     )?;
-    builder.set_certificate_chain_file(&self.certificate_chain_file_name)?;
+    builder.set_certificate_file(
+      &self.ca_file_name,
+      openssl::ssl::SslFiletype::PEM,
+    )?;
     builder.check_private_key()?;
 
     self.ssl_acceptor = Arc::new(builder.build());
@@ -457,7 +510,10 @@ impl Router {
   ///     .set_private_key_file("windmark_private.pem", ssl::SslFiletype::PEM)
   ///     .unwrap();
   ///   builder
-  ///     .set_certificate_chain_file("windmark_pair.pem")
+  ///     .set_certificate_file(
+  ///       "windmark_public.pem",
+  ///       openssl::ssl::SslFiletype::PEM,
+  ///     )
   ///     .unwrap();
   ///   builder.check_private_key().unwrap();
   ///
@@ -562,17 +618,19 @@ impl Router {
     self
   }
 
-  /// Attach a module to a `Router`.
+  /// Attach a stateless module to a `Router`.
   ///
   /// A module is an extension or middleware to a `Router`. Modules get full
   /// access to the `Router`, but can be extended by a third party.
   ///
   /// # Examples
   ///
+  /// ## Integrated Module
+  ///
   /// ```rust
   /// use windmark::Response;
   ///
-  /// windmark::Router::new().attach(|r| {
+  /// windmark::Router::new().attach_stateless(|r| {
   ///   r.mount(
   ///     "/module",
   ///     Box::new(|_| Response::Success("This is a module!".into())),
@@ -584,13 +642,89 @@ impl Router {
   ///   }));
   /// });
   /// ```
-  pub fn attach<F>(&mut self, mut module: F) -> &mut Self
+  ///
+  /// ## External Module
+  ///
+  /// ```rust
+  /// use windmark::Response;
+  ///
+  /// mod windmark_example {
+  ///   pub fn module(router: &mut windmark::Router) {
+  ///     router.mount(
+  ///       "/module",
+  ///       Box::new(|_| windmark::Response::Success("This is a module!".into())),
+  ///     );
+  ///   }
+  /// }
+  ///
+  /// windmark::Router::new().attach_stateless(windmark_example::module);
+  /// ```
+  pub fn attach_stateless<F>(&mut self, mut module: F) -> &mut Self
   where F: FnMut(&mut Self) {
     module(self);
 
     self
   }
 
+  /// Attach a stateful module to a `Router`.
+  ///
+  /// Like a stateless module is an extension or middleware to a `Router`.
+  /// Modules get full access to the `Router` and can be extended by a third
+  /// party, but also, can create hooks will be executed through various parts
+  /// of a routes' lifecycle. Stateful modules also have state, so variables can
+  /// be stored for further access.
+  ///
+  /// # Panics
+  ///
+  /// May panic if the stateful module cannot be attached.
+  ///
+  /// # Examples
+  ///
+  /// ```rust
+  /// use log::info;
+  /// use windmark::{returnable::CallbackContext, Response, Router};
+  ///
+  /// #[derive(Default)]
+  /// struct Clicker {
+  ///   clicks: isize,
+  /// }
+  /// impl windmark::Module for Clicker {
+  ///   fn on_attach(&mut self, _: &mut Router) {
+  ///     info!("clicker has been attached!");
+  ///   }
+  ///
+  ///   fn on_pre_route(&mut self, context: CallbackContext<'_>) {
+  ///     self.clicks += 1;
+  ///
+  ///     info!(
+  ///       "clicker has been called pre-route on {} with {} clicks!",
+  ///       context.url.path(),
+  ///       self.clicks
+  ///     );
+  ///   }
+  ///
+  ///   fn on_post_route(&mut self, context: CallbackContext<'_>) {
+  ///     info!(
+  ///       "clicker has been called post-route on {} with {} clicks!",
+  ///       context.url.path(),
+  ///       self.clicks
+  ///     );
+  ///   }
+  /// }
+  ///
+  /// Router::new().attach(Clicker::default());
+  /// ```
+  pub fn attach(
+    &mut self,
+    mut module: impl Module + 'static + Send,
+  ) -> &mut Self {
+    module.on_attach(self);
+
+    (*self.modules.lock().unwrap()).push(Box::new(module));
+
+    self
+  }
+
   /// Specify a custom character set.
   ///
   /// Defaults to `"utf-8"`.
@@ -620,6 +754,21 @@ impl Router {
 
     self
   }
+
+  /// Specify a custom port.
+  ///
+  /// Defaults to `1965`.
+  ///
+  /// # Examples
+  ///
+  /// ```rust
+  /// windmark::Router::new().set_port(1965); 
+  /// ```
+  pub fn set_port(&mut self, port: i32) -> &mut Self {
+    self.port = port;
+
+    self
+  }
 }
 impl Default for Router {
   fn default() -> Self {
@@ -631,7 +780,7 @@ impl Default for Router {
         )
       }))),
       private_key_file_name: "".to_string(),
-      certificate_chain_file_name: "".to_string(),
+      ca_file_name: "".to_string(),
       header: Arc::new(Mutex::new(Box::new(|_| "".to_string()))),
       footer: Arc::new(Mutex::new(Box::new(|_| "".to_string()))),
       ssl_acceptor: Arc::new(
@@ -645,6 +794,8 @@ impl Default for Router {
       post_route_callback: Arc::new(Mutex::new(Box::new(|_, _, _| {}))),
       charset: "utf-8".to_string(),
       language: "en".to_string(),
+      port: 1965,
+      modules: Arc::new(Mutex::new(vec![])),
     }
   }
 }
